@@ -1,4 +1,6 @@
 """Computation of reductions on vectors."""
+from __future__ import annotations
+
 
 __copyright__ = "Copyright (C) 2010 Andreas Kloeckner"
 
@@ -28,15 +30,25 @@ Based on code/ideas by Mark Harris <mharris@nvidia.com>.
 None of the original source code remains.
 """
 
+import builtins
 from dataclasses import dataclass
-from typing import Any, List, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Literal, cast, overload
 
 import numpy as np
 
 import pyopencl as cl
+import pyopencl.array as cl_array
 from pyopencl.tools import (
-    DtypedArgument, KernelTemplateBase, _process_code_for_macro,
-    context_dependent_memoize, dtype_to_ctype)
+    DtypedArgument,
+    KernelTemplateBase,
+    _process_code_for_macro,
+    context_dependent_memoize,
+    dtype_to_ctype,
+)
+
+
+if TYPE_CHECKING:
+    from pyopencl.typing import Allocator
 
 
 # {{{ kernel source
@@ -129,7 +141,7 @@ class _ReductionInfo:
 
     program: cl.Program
     kernel: cl.Kernel
-    arg_types: List[DtypedArgument]
+    arg_types: list[DtypedArgument]
 
 
 def _get_reduction_source(
@@ -139,12 +151,12 @@ def _get_reduction_source(
         neutral: str,
         reduce_expr: str,
         map_expr: str,
-        parsed_args: List[DtypedArgument],
+        parsed_args: list[DtypedArgument],
         name: str = "reduce_kernel",
         preamble: str = "",
         arg_prep: str = "",
-        device: Optional[cl.Device] = None,
-        max_group_size: Optional[int] = None) -> Tuple[str, int]:
+        device: cl.Device | None = None,
+        max_group_size: int | None = None) -> tuple[str, int]:
 
     if device is not None:
         devices = [device]
@@ -205,13 +217,13 @@ def get_reduction_kernel(
         dtype_out: Any,
         neutral: str,
         reduce_expr: str,
-        map_expr: Optional[str] = None,
-        arguments: Optional[List[DtypedArgument]] = None,
+        map_expr: str | None = None,
+        arguments: list[DtypedArgument] | None = None,
         name: str = "reduce_kernel",
         preamble: str = "",
-        device: Optional[cl.Device] = None,
+        device: cl.Device | None = None,
         options: Any = None,
-        max_group_size: Optional[int] = None) -> _ReductionInfo:
+        max_group_size: int | None = None) -> _ReductionInfo:
     if stage not in (1, 2):
         raise ValueError(f"unknown stage index: '{stage}'")
 
@@ -219,8 +231,11 @@ def get_reduction_kernel(
         map_expr = "pyopencl_reduction_inp[i]" if stage == 2 else "in[i]"
 
     from pyopencl.tools import (
-        VectorArg, get_arg_list_scalar_arg_dtypes, get_arg_offset_adjuster_code,
-        parse_arg_list)
+        VectorArg,
+        get_arg_list_scalar_arg_dtypes,
+        get_arg_offset_adjuster_code,
+        parse_arg_list,
+    )
 
     if arguments is None:
         raise ValueError("arguments must not be None")
@@ -229,9 +244,9 @@ def get_reduction_kernel(
     arg_prep = get_arg_offset_adjuster_code(arguments)
 
     if stage == 2 and arguments is not None:
-        arguments = (
-                [VectorArg(dtype_out, "pyopencl_reduction_inp")]
-                + arguments)
+        arguments = [
+                VectorArg(dtype_out, "pyopencl_reduction_inp"),
+                *arguments]
 
     source, group_size = _get_reduction_source(
             ctx, dtype_to_ctype(dtype_out), dtype_out.itemsize,
@@ -301,8 +316,8 @@ class ReductionKernel:
             dtype_out: Any,
             neutral: str,
             reduce_expr: str,
-            map_expr: Optional[str] = None,
-            arguments: Optional[Union[str, List[DtypedArgument]]] = None,
+            map_expr: str | None = None,
+            arguments: str | list[DtypedArgument] | None = None,
             name: str = "reduce_kernel",
             options: Any = None,
             preamble: str = "") -> None:
@@ -342,7 +357,40 @@ class ReductionKernel:
                 name=f"{name}_stage2", options=options, preamble=preamble,
                 max_group_size=max_group_size)
 
-    def __call__(self, *args: Any, **kwargs: Any) -> cl.Event:
+    @overload
+    def __call__(self,
+                *args: object,
+                return_event: Literal[True],
+                queue: cl.CommandQueue | None = None,
+                allocator: Allocator | None = None,
+                wait_for: cl.WaitList = None,
+                out: cl_array.Array | None = None,
+                range: slice | None = None,
+                slice: slice | None = None
+            ) ->  tuple[cl_array.Array, cl.Event]: ...
+
+    @overload
+    def __call__(self,
+                *args: object,
+                return_event: Literal[False],
+                queue: cl.CommandQueue | None = None,
+                allocator: Allocator | None = None,
+                wait_for: cl.WaitList = None,
+                out: cl_array.Array | None = None,
+                range: slice | None = None,
+                slice: slice | None = None
+            ) -> cl_array.Array: ...
+
+    def __call__(self,
+                *args: object,
+                return_event: bool = False,
+                queue: cl.CommandQueue | None = None,
+                allocator: Allocator | None = None,
+                wait_for: cl.WaitList = None,
+                out: cl_array.Array | None = None,
+                range: slice | None = None,
+                slice: slice | None = None
+            ) -> cl_array.Array | tuple[cl_array.Array, cl.Event]:
         """Invoke the generated kernel.
 
         |explain-waitfor|
@@ -381,18 +429,6 @@ class ReductionKernel:
             ``(scalar_array, event)``.
         """
 
-        queue = kwargs.pop("queue", None)
-        allocator = kwargs.pop("allocator", None)
-        wait_for = kwargs.pop("wait_for", None)
-        return_event = kwargs.pop("return_event", False)
-        out = kwargs.pop("out", None)
-
-        range_ = kwargs.pop("range", None)
-        slice_ = kwargs.pop("slice", None)
-
-        if kwargs:
-            raise TypeError("invalid keyword argument to reduction kernel")
-
         if wait_for is None:
             wait_for = []
         else:
@@ -406,13 +442,14 @@ class ReductionKernel:
 
         while True:
             invocation_args = []
-            vectors = []
+            vectors: list[cl_array.Array] = []
 
             array_empty = empty
 
             from pyopencl.tools import VectorArg
-            for arg, arg_tp in zip(args, stage_inf.arg_types):
+            for arg, arg_tp in zip(args, stage_inf.arg_types, strict=True):
                 if isinstance(arg_tp, VectorArg):
+                    assert isinstance(arg, cl_array.Array)
                     array_empty = arg.__class__
                     if not arg.flags.forc:
                         raise RuntimeError(
@@ -434,31 +471,30 @@ class ReductionKernel:
 
             # {{{ range/slice processing
 
-            if range_ is not None:
-                if slice_ is not None:
+            if range is not None:
+                if slice is not None:
                     raise TypeError("may not specify both range and slice "
                             "keyword arguments")
 
             else:
-                if slice_ is None:
-                    slice_ = slice(None)
+                if slice is None:
+                    slice = builtins.slice(None)
 
                 if repr_vec is None:
                     raise TypeError(
                             "must have vector argument when range is not specified")
 
-                range_ = slice(*slice_.indices(repr_vec.size))
+                range = builtins.slice(*slice.indices(repr_vec.size))
 
-            assert range_ is not None
+            assert range is not None
 
-            start = range_.start
+            start = cast("int | None", range.start)
             if start is None:
                 start = 0
-            if range_.step is None:
+            step = cast("int | None", range.step)
+            if step is None:
                 step = 1
-            else:
-                step = range_.step
-            sz = abs(range_.stop - start)//step
+            sz = abs(cast("int", range.stop) - start) //step
 
             # }}}
 
@@ -495,7 +531,7 @@ class ReductionKernel:
                 macrogroup_size = group_count*stage_inf.group_size
                 seq_count = (sz + macrogroup_size - 1) // macrogroup_size
 
-            size_args = [start, step, range_.stop, seq_count, sz]
+            size_args = [start, step, range.stop, seq_count, sz]
 
             if group_count == 1 and out is not None:
                 result = out
@@ -512,8 +548,7 @@ class ReductionKernel:
                     use_queue,
                     (group_count*stage_inf.group_size,),
                     (stage_inf.group_size,),
-                    *([result.base_data, result.offset]
-                        + invocation_args + size_args),
+                    *([result.base_data, result.offset, *invocation_args, *size_args]),
                     wait_for=wait_for)
             wait_for = [last_evt]
 
@@ -526,9 +561,9 @@ class ReductionKernel:
                     return result
             else:
                 stage_inf = self.stage_2_inf
-                args = (result,) + stage1_args
+                args = (result, *stage1_args)
 
-                range_ = slice_ = None
+                range = slice = None
 
 # }}}
 
@@ -538,12 +573,12 @@ class ReductionKernel:
 class ReductionTemplate(KernelTemplateBase):
     def __init__(
             self,
-            arguments: Union[str, List[DtypedArgument]],
+            arguments: str | list[DtypedArgument],
             neutral: str,
             reduce_expr: str,
-            map_expr: Optional[str] = None,
-            is_segment_start_expr: Optional[str] = None,
-            input_fetch_exprs: Optional[List[Tuple[str, str, int]]] = None,
+            map_expr: str | None = None,
+            is_segment_start_expr: str | None = None,
+            input_fetch_exprs: list[tuple[str, str, int]] | None = None,
             name_prefix: str = "reduce",
             preamble: str = "",
             template_processor: Any = None) -> None:
